<?php

namespace App\Entity;

use App\Repository\TagRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\Mapping as ORM;
use Gedmo\Mapping\Annotation as Gedmo;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * A tag
 *
 * Tags are by user and a “materialized path” tree structure
 *
 * @link https://github.com/doctrine-extensions/DoctrineExtensions/blob/main/doc/tree.md#materialized-path
 */
#[ORM\Entity(repositoryClass: TagRepository::class)]
#[Gedmo\Tree(type: 'materializedPath')]
class Tag {
    use TEntityBase;

    /**
     * @var User The owner
     */
    #[ORM\ManyToOne(inversedBy: 'tags')]
    #[ORM\JoinColumn(nullable: false, onDelete: 'CASCADE')]
    #[Assert\Valid]
    private User $owner;

    /**
     * @var string The name
     */
    #[ORM\Column(length: 50)]
    #[Gedmo\TreePathSource]
    #[Assert\NotBlank]
    private string $name;

    /**
     * @var string The path (complete name)
     */
    #[ORM\Column(type: Types::TEXT)]
    #[Gedmo\TreePath]
    #[Assert\NotBlank]
    private string $path;

    /**
     * @var Tag|null The parent tag
     */
    #[ORM\ManyToOne(targetEntity: self::class, inversedBy: 'children')]
    #[ORM\JoinColumn(onDelete: 'CASCADE')]
    #[Gedmo\TreeParent]
    private ?self $parent = null;
    /**
     * @var Collection<int, self> The children tag
     */
    #[ORM\OneToMany(mappedBy: 'parent', targetEntity: self::class)]
    private Collection $children;

    /**
     * @var Collection<int, Document> The {@see Document documents}
     */
    #[ORM\ManyToMany(targetEntity: Document::class, mappedBy: 'tags')]
    private Collection $documents;

    /**
     * Initialization
     */
    public function __construct () {
        $this->children = new ArrayCollection();
        $this->documents = new ArrayCollection();
    }

    /**
     * The owner
     *
     * @return User The owner
     */
    public function getOwner (): User {
        return $this->owner;
    }
    /**
     * Change the owner
     *
     * @param User $owner The new owner
     *
     * @return $this
     */
    public function setOwner (User $owner): static {
        $this->owner = $owner;

        return $this;
    }

    /**
     * The name
     *
     * @return string The name
     */
    public function getName (): string {
        return $this->name;
    }
    /**
     * Change the name
     *
     * @param string $name The new name
     *
     * @return $this
     */
    public function setName (string $name): self {
        $this->name = $name;

        return $this;
    }

    /**
     * The path (complete name)
     *
     * @return string The path (complete name)
     */
    public function getPath (): string
    {
        return $this->path;
    }
    /**
     * Change the path (complete name)
     *
     * @param string $path The new path (complete name)
     *
     * @return $this
     */
    public function setPath (string $path): static
    {
        $this->path = $path;

        return $this;
    }

    /**
     * The parent tag
     *
     * @return Tag|null The parent tag
     */
    public function getParent (): ?Tag {
        return $this->parent;
    }
    /**
     * Change the parent tag
     *
     * @param Tag|null $parent The new parent tag
     *
     * @return $this
     */
    public function setParent (?Tag $parent): self {
        $this->parent = $parent;
        return $this;
    }

    /**
     * The children tag
     *
     * @return Collection<int, self> The children tag
     */
    public function getChildren (): Collection {
        return $this->children;
    }

    /**
     * The {@see Document documents}
     *
     * @return Collection<int, Document> The {@see Document documents}
     */
    public function getDocuments (): Collection {
        return $this->documents;
    }
    /**
     * Add a {@see Document document}
     *
     * Do nothing if the document is already present
     *
     * @param Document $document The {@see Document document} to add
     *
     * @return $this
     */
    public function addDocument (Document $document): static {
        if (!$this->documents->contains($document)) {
            $this->documents->add($document);
            $document->addTag($this);
        }

        return $this;
    }
    /**
     * Remove a {@see Document document}
     *
     * @param Document $document The {@see Document document} to remove
     *
     * @return $this
     */
    public function removeDocument (Document $document): static {
        if ($this->documents->removeElement($document)) {
            $document->removeTag($this);
        }

        return $this;
    }
}
